@@ -19,8 +19,13 @@ pub use streaming::StreamingBody;
 /// Read and write operations to a [`Body`] are automatically buffered, though you can take direct
 /// control over aspects of the buffering using the [`BufRead`] methods and [`Write::flush()`].
 pub struct Body {
-    reader: BufReader<BodyHandle>,
-    writer: BufWriter<BodyHandleWrapper>,
+    // NOTE: The order of these fields with these different handles is load
+    // bearing. `BufWriter` needs `BodyHandle` so that it flushes out the buffer
+    // and then drops the `BodyHandle` out properly when `Body` is dropped.
+    // `BodyHandleWrapper` makes sure we don't double free the memory that
+    // `BodyHandle` points too.
+    reader: BufReader<BodyHandleWrapper>,
+    writer: BufWriter<BodyHandle>,
 }
 
 impl Debug for Body {
@@ -38,14 +43,16 @@ impl Body {
 
     // this is not exported, since misuse can lead to data getting dropped or appearing out of order
     fn handle(&mut self) -> &mut BodyHandle {
-        self.reader.get_mut()
+        self.writer.get_mut()
     }
 
     /// Convert a [`Body`] into the low-level [`BodyHandle`] interface.
     pub fn into_handle(mut self) -> BodyHandle {
         self.put_back_read_buf();
-        self.writer.flush().expect("fastly_http_body::write failed");
-        self.reader.into_inner()
+        // Flushes the buffer and returns the underlying `BodyHandle`
+        self.writer
+            .into_inner()
+            .expect("fastly_http_body::write failed")
     }
 
     /// Put any currently buffered read data back at the front of the body.
@@ -56,7 +63,7 @@ impl Body {
             // buffer is borrowed. Since we're not going to read or write through the `self`
             // interface while `body_handle` is live, no other aliases of the handle will be used.
             let mut body_handle =
-                ManuallyDrop::new(unsafe { BodyHandle::from_u32(self.reader.get_ref().as_u32()) });
+                ManuallyDrop::new(unsafe { BodyHandle::from_u32(self.writer.get_ref().as_u32()) });
             let nwritten = body_handle.write_front(read_buf);
             drop(read_buf);
             // Let the `BufReader` know that we've consumed these bytes from its internal buffer so
@@ -388,8 +395,8 @@ impl From<BodyHandle> for Body {
         // public interface
         let handle2 = unsafe { BodyHandle::from_u32(handle.as_u32()) };
         Self {
-            reader: BufReader::new(handle),
-            writer: BufWriter::new(BodyHandleWrapper::new(handle2)),
+            reader: BufReader::new(BodyHandleWrapper::new(handle)),
+            writer: BufWriter::new(handle2),
         }
     }
 }
@@ -544,11 +551,10 @@ struct BodyHandleWrapper {
 
 impl Drop for BodyHandleWrapper {
     fn drop(&mut self) {
-        // Weirdly having the handle field be ManuallyDrop<BodyHandle> was
-        // causing issues so we replace it with an invalid handle which will not
-        // drop the resource in BodyHandle's Drop impl and then we wrap the
-        // actual handle we get back with ManuallyDrop which will just be a u32
-        // on the stack that will get cleared out so we don't leak any memory
+        // In order to avoid doing a double free we replace the handle with an
+        // invalid handle that doesn't point to anything. We then `mem::forget`
+        // the original handle which is just a wrapper around `u32` and is left
+        // on the stack that will get cleared out so we don't leak any memory.
         let handle = mem::replace(&mut self.handle, unsafe {
             BodyHandle::from_u32(fastly_shared::INVALID_BODY_HANDLE)
         });
@@ -560,10 +566,6 @@ impl BodyHandleWrapper {
     fn new(handle: BodyHandle) -> Self {
         Self { handle }
     }
-
-    fn write_front(&mut self, buf: &[u8]) -> usize {
-        self.handle.write_front(buf)
-    }
 }
 
 impl Write for BodyHandleWrapper {
@@ -579,3 +581,9 @@ impl Write for BodyHandleWrapper {
         self.handle.flush()
     }
 }
+
+impl Read for BodyHandleWrapper {
+    fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
+        self.handle.read(buf)
+    }
+}
