@@ -1,5 +1,4 @@
 use super::super::handle::BodyHandle;
-use crate::abi;
 use std::io::Write;
 
 /// A low-level interface to a streaming HTTP body.
@@ -17,7 +16,7 @@ use std::io::Write;
 #[derive(Debug, Eq, Hash, PartialEq)]
 #[repr(transparent)]
 pub struct StreamingBodyHandle {
-    handle: u32,
+    handle: BodyHandle,
 }
 
 impl StreamingBodyHandle {
@@ -25,27 +24,15 @@ impl StreamingBodyHandle {
     ///
     /// This should only be used when calling the raw ABI directly, and care should be taken not to
     /// reuse or alias handle values.
-    pub(crate) fn from_body_handle(body_handle: &BodyHandle) -> Self {
+    pub(crate) fn from_body_handle(body_handle: BodyHandle) -> Self {
         Self {
-            handle: body_handle.handle,
-        }
-    }
-
-    /// Get a non-streaming body handle from a streaming handle.
-    ///
-    /// This should only be used when calling the raw ABI directly, and care should be taken not to
-    /// reuse or alias handle values, or to use the returned value in contexts where a streaming
-    /// handle is not valid.
-    fn as_body_handle(&self) -> BodyHandle {
-        BodyHandle {
-            handle: self.handle,
+            handle: body_handle,
         }
     }
 
     /// Append another body onto the end of this body.
     ///
-    /// This operation is performed in amortized constant time, and so should always be preferred to
-    /// reading an entire body and then writing the same contents to another body.
+    #[doc = include_str!("../../../../docs/snippets/body-append-constant-time.md")]
     ///
     /// The other body will no longer be valid after this call.
     ///
@@ -59,7 +46,7 @@ impl StreamingBodyHandle {
     /// streaming_body.append(other_body);
     /// ```
     pub fn append(&mut self, other: BodyHandle) {
-        self.as_body_handle().append(other)
+        self.handle.append(other)
     }
 
     /// Write a slice of bytes to the end of this streaming body, and return the number of bytes written.
@@ -73,7 +60,7 @@ impl StreamingBodyHandle {
     /// streaming_body.write_bytes(&[0, 1, 2, 3]);
     /// ```
     pub fn write_bytes(&mut self, bytes: &[u8]) -> usize {
-        self.as_body_handle().write_bytes(bytes)
+        self.handle.write_bytes(bytes)
     }
 
     /// Write a string slice to the end of this streaming body, and return the number of bytes written.
@@ -91,14 +78,6 @@ impl StreamingBodyHandle {
     }
 }
 
-impl Drop for StreamingBodyHandle {
-    fn drop(&mut self) {
-        unsafe { abi::fastly_http_body::close(self.as_body_handle().as_u32()) }
-            .result()
-            .expect("fastly_http_body::close failed");
-    }
-}
-
 impl Write for StreamingBodyHandle {
     fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
         Ok(self.write_bytes(buf))
